from _typeshed import Incomplete

from .numbers import NUMERIC_TYPES as NUMERIC_TYPES
from .strings import safe_string as safe_string

class DummyCode: ...

string_types: Incomplete

def deprecated(reason): ...
